@@ -1,7 +1,7 @@
 package Dist::Zilla::Plugin::Sah::Schemas;
 
-our $DATE = '2016-05-08'; # DATE
-our $VERSION = '0.001'; # VERSION
+our $DATE = '2016-07-20'; # DATE
+our $VERSION = '0.002'; # VERSION
 
 use 5.010001;
 use strict;
@@ -15,6 +15,7 @@ with (
     'Dist::Zilla::Role::FileFinderUser' => {
         default_finders => [':InstallModules'],
     },
+    'Dist::Zilla::Role::PrereqSource',
 );
 
 sub munge_files {
@@ -22,6 +23,9 @@ sub munge_files {
 
     my $self = shift;
 
+    $self->{_our_schema_modules}  //= {};
+    $self->{_used_schema_modules} //= {};
+
     for my $file (@{ $self->found_files }) {
         unless ($file->isa("Dist::Zilla::File::OnDisk")) {
             $self->log_debug(["skipping %s: not an ondisk file, currently generated file is assumed to be OK", $file->name]);
@@ -37,20 +41,59 @@ sub munge_files {
                 {
                     local @INC = ("lib", @INC);
                     require $package_pm;
+                    $self->{_our_schema_modules}{$package}++;
                 }
                 my $sch = ${"$package\::schema"};
+                my $nsch = Data::Sah::Normalize::normalize_schema($sch);
 
                 # check that schema is already normalized
                 {
                     require Data::Dump;
                     require Data::Sah::Normalize;
                     require Text::Diff;
-                    my $nsch = Data::Sah::Normalize::normalize_schema($sch);
                     my $sch_dmp  = Data::Dump::dump($sch);
                     my $nsch_dmp = Data::Dump::dump($nsch);
                     last if $sch_dmp eq $nsch_dmp;
                     my $diff = Text::Diff::diff(\$sch_dmp, \$nsch_dmp);
-                    $self->log_fatal("Schema in $file_name is not normalized, below is the dump diff: " . $diff);
+                    $self->log_fatal("Schema in $file_name is not normalized, below is the dump diff (- is current, + is normalized): " . $diff);
+                }
+
+                # collect other Sah::Schema::* modules that are used, this will
+                # be added as prereq
+              COLLECT_BASE_SCHEMAS:
+                {
+                    require Data::Sah::Normalize;
+                    require Data::Sah::Resolve;
+                    require Data::Sah::Util::Subschema;
+
+                    $self->log_debug(["Finding schema modules required by %s", $package]);
+
+                    my $subschemas;
+                    eval {
+                        $subschemas = Data::Sah::Util::Subschema::extract_subschemas(
+                            {schema_is_normalized => 1},
+                            $nsch,
+                        );
+                    };
+                    if ($@) {
+                        $self->log(["Can't extract subschemas from schema in %s (%s), skipped", $package, $@]);
+                        last COLLECT_BASE_SCHEMAS;
+                    }
+
+                    for my $subsch ($nsch, @$subschemas) {
+                        my $nsubsch = Data::Sah::Normalize::normalize_schema($subsch);
+                        my $res = Data::Sah::Resolve::resolve_schema(
+                            {
+                                schema_is_normalized => 1,
+                                return_intermediates => 1,
+                            },
+                            $nsubsch);
+                        my $intermediates = $res->[2];
+                        for my $i (0..$#{$intermediates}-1) {
+                            my $mod = "Sah::Schema::$intermediates->[$i]";
+                            $self->{_used_schema_modules}{$mod}++;
+                        }
+                    }
                 }
 
                 # set ABSTRACT from schema's summary
@@ -75,6 +118,17 @@ sub munge_files {
     } # for $file
 }
 
+sub register_prereqs {
+    my $self = shift;
+
+    #use DD; dd $self->{_used_schema_modules}; dd $self->{_our_schema_modules};
+    for my $mod (sort keys %{$self->{_used_schema_modules}}) {
+        next if $self->{_our_schema_modules}{$mod};
+        $self->log(["Adding prereq to %s", $mod]);
+        $self->zilla->register_prereqs({phase=>'runtime'}, $mod);
+    }
+}
+
 __PACKAGE__->meta->make_immutable;
 1;
 # ABSTRACT: Plugin to use when building Sah-Schemas-* distribution
@@ -91,7 +145,7 @@ Dist::Zilla::Plugin::Sah::Schemas - Plugin to use when building Sah-Schemas-* di
 
 =head1 VERSION
 
-This document describes version 0.001 of Dist::Zilla::Plugin::Sah::Schemas (from Perl distribution Dist-Zilla-Plugin-Sah-Schemas), released on 2016-05-08.
+This document describes version 0.002 of Dist::Zilla::Plugin::Sah::Schemas (from Perl distribution Dist-Zilla-Plugin-Sah-Schemas), released on 2016-07-20.
 
 =head1 SYNOPSIS
 
@@ -120,6 +174,8 @@ Otherwise, the build is aborted.
 
 =item * Set module abstract from the schema's summary
 
+=item * Add a prereq to other Sah::Schema::* module if schema depends on those other schemas
+
 =back
 
 =for Pod::Coverage .+
